local addonName, addonTable = ...
local L = addonTable.L
--------------------------------------
-- new widget function
--------------------------------------
-- CheckButton
local function newCheckButton(frame, label, tooltip, onClickFn)
	local cb = CreateFrame("CheckButton", addonName .. "CheckButton_" .. label, frame, "InterfaceOptionsCheckButtonTemplate")
	cb:SetScript("OnClick", function(self)
		PlaySound(self:GetChecked() and "igMainMenuOptionCheckBoxOn" or "igMainMenuOptionCheckBoxOff")
		onClickFn(self, self:GetChecked() and true or false) 
	end)
	cb.label = _G[cb:GetName() .. "Text"]
	cb.label:SetText(label)
	cb.tooltipText = label
	cb.tooltipRequirement = tooltip
	return cb
end

-- Slider
local function newSlider(frame, label, low, high, tooltip, OnValueChangedFn)
	local slider = CreateFrame("Slider", addonName .. "Slider_" .. label, frame, "OptionsSliderTemplate")
    slider:SetMinMaxValues(low, high)
    slider:SetValueStep(1)
	slider:SetObeyStepOnDrag(true)
    slider.text = _G[slider:GetName() .. "Text"]
    slider.low = _G[slider:GetName() .. "Low"]
    slider.high = _G[slider:GetName() .. "High"]

    slider.text:SetText(label)
    slider.low:SetText("")
    slider.high:SetText("")
    if tooltip then slider.tooltipText = tooltip end
	
	local vText = slider:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	vText:SetHeight(35)
    vText:SetPoint("LEFT", slider, "RIGHT", 5, 0)

    slider:SetScript("OnValueChanged", function(self, value)
		vText:SetText(value)
        OnValueChangedFn(self, value)
    end)
	
	return slider
end
--------------------------------------
-- Config Panel
--------------------------------------
local FSFConfigPanel = addonTable.frame
FSFConfigPanel.name = addonName
FSFConfigPanel:Hide()
FSFConfigPanel:SetScript("OnShow", function(frame)
	frame:SetScript("OnShow", nil)
	
	-- text
    local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 16, -16)
    title:SetText(addonName)
	
	local desc = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    desc:SetHeight(35)
    desc:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
    desc:SetText(L.DESC .. " (" .. addonTable.cfg.fixed .. " fixed)")
	
	-- check button
	local fixAll, fixSpecific
	
	fixAll = newCheckButton(
		frame, L.FIX_ALL, L.FIX_ALL_DESC,
		function(self, value) 
			addonTable.cfg.fixAll = value
			fixSpecific:SetChecked(not value)
		end)
	fixAll:SetChecked(addonTable.cfg.fixAll)
	fixAll:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -2, -116)
	
	fixSpecific = newCheckButton(
		frame, L.FIX_SPECIFIC, L.FIX_SPECIFIC_DESC,
		function(self, value)
			addonTable.cfg.fixAll = not value 
			fixAll:SetChecked(not value)
		end)
	fixSpecific:SetChecked(not addonTable.cfg.fixAll)
	fixSpecific:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -2, -146)
	
	-- slider
	local xslider = newSlider(
		frame, L.X_OFFSET, -5, 5, nil,
		function(self, value) addonTable.cfg.xoffset = value end)
	xslider:SetValue(addonTable.cfg.xoffset)
	xslider:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -70)
	
	local yslider = newSlider(
		frame, L.Y_OFFSET, -5, 5, nil,
		function(self, value) addonTable.cfg.yoffset = value end)
	yslider:SetValue(addonTable.cfg.yoffset)
	yslider:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 200, -70)
	
	-- button
	local edit = CreateFrame("Button", addonName .. "Button_Edit", frame, "UIPanelButtonTemplate")
	edit:SetText(L.EDIT)
	edit:SetWidth(100)
	edit:SetHeight(24)
	edit:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -200)
	edit:SetScript("OnClick", function()
		addonTable.ShowDialog(addonTable.load())
	end)
	edit.tooltipText = L.EDIT_DESC
	
	local reset = CreateFrame("Button", addonName .. "Button_Reset", frame, "UIPanelButtonTemplate")
	reset:SetText(L.RESET)
	reset:SetWidth(100)
	reset:SetHeight(24)
	reset:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 120, -200)
	reset:SetScript("OnClick", function()
		addonTable.reset()
	end)
	reset.tooltipText = L.RESET_DESC
	
	local reload = CreateFrame("Button", addonName .. "Button_Reload", frame, "UIPanelButtonTemplate")
	reload:SetText(L.RELOAD)
	reload:SetWidth(100)
	reload:SetHeight(24)
	reload:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 240, -200)
	reload:SetScript("OnClick", function()
		_G.ReloadUI()
	end)
end)

InterfaceOptions_AddCategory(FSFConfigPanel)