-- fyhcslb 10/29/2016

local _, addonTable = ...
--------------------------------------
-- Functions
--------------------------------------
local function SetShadowOffset(font, xoffset, yoffset)
	if pcall(function() font:GetShadowOffset() end) then -- Attempt to access forbidden object from code tainted by an AddOn :(
		x, y = font:GetShadowOffset()
		if (x ~= 0 and y ~= 0) and (x ~= xoffset or y ~= yoffset) then
			font:SetShadowOffset(xoffset, yoffset)
			return true
		end
	else
		return false
	end
end

local function FixFontInstance(refix, fontInstances, xoffset, yoffset)
	if refix then
		for name, obj in pairs(_G) do
			if type(obj) == "table" and SetShadowOffset(obj, xoffset, yoffset) then
				table.insert(fontInstances, name)
			end
		end
	else
		for _, name in pairs(fontInstances) do
			SetShadowOffset(_G[name], xoffset, yoffset)
		end
		-- fontInstances
	end
	FontShadowFixConfig.fixed = #fontInstances
end

local function Load()
	local temp = ""
	for _, v in pairs(addonTable.cfg.specificFonts) do
		if v and v ~= "" then
			temp = temp .. v .. "\n"
		end
	end
	return temp
end

local function Save(fonts)
	wipe(FontShadowFixConfig.fixedFonts)
	if fonts and fonts ~= "" then
		wipe(FontShadowFixConfig.specificFonts)
		for font in string.gmatch(fonts, "[a-zA-Z0-9_]*") do
			if _G[font] then
				table.insert(FontShadowFixConfig.specificFonts, font)
			end
		end
		FontShadowFixConfig.fixAll = false
		_G.ReloadUI()
	end
end

local function Reset()
	wipe(FontShadowFixConfig)
	-- FontShadowFixConfig.fixAll = true
	-- FontShadowFixConfig.fixed = 0
	-- FontShadowFixConfig.xoffset = 1
	-- FontShadowFixConfig.yoffset = -3
	-- wipe(FontShadowFixConfig.fixedFonts)
	-- wipe(FontShadowFixConfig.specificFonts)
	_G.ReloadUI()
end

addonTable.load = Load
addonTable.save = Save
addonTable.reset = Reset

--------------------------------------
-- Event
--------------------------------------
local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGIN")
addonTable.frame = frame

function frame:ADDON_LOADED(arg1)
	if arg1 == "FontShadowFix" then
		frame:UnregisterEvent("ADDON_LOADED")
		if type(FontShadowFixConfig) ~= "table" then FontShadowFixConfig = {} end
		if type(FontShadowFixConfig.fixAll) ~= "boolean" then FontShadowFixConfig.fixAll = true end
		if type(FontShadowFixConfig.fixed) ~= "number" then FontShadowFixConfig.fixed = 0 end
		if type(FontShadowFixConfig.xoffset) ~= "number" then FontShadowFixConfig.xoffset = 1 end
		if type(FontShadowFixConfig.yoffset) ~= "number" then FontShadowFixConfig.yoffset = -3 end
		if type(FontShadowFixConfig.fixedFonts) ~= "table" then FontShadowFixConfig.fixedFonts = {} end
		if type(FontShadowFixConfig.specificFonts) ~= "table" then FontShadowFixConfig.specificFonts = {} end
		
		addonTable.cfg = FontShadowFixConfig
	end
end

function frame:PLAYER_LOGIN()
	if FontShadowFixConfig.fixAll and #FontShadowFixConfig.fixedFonts == 0 then
		-- print("fix all")
		FixFontInstance(true, FontShadowFixConfig.fixedFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset)
	elseif not FontShadowFixConfig.fixAll then
		-- print("fix specific")
		FixFontInstance(false, FontShadowFixConfig.specificFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset)
	else
		-- print("fix all(scaned)")
		FixFontInstance(false, FontShadowFixConfig.fixedFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset)
	end
end

frame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

--------------------------------------
-- Slash
--------------------------------------
-- SLASH_FONTSHADOWFIX1 = "/fsf"
-- function SlashCmdList.FONTSHADOWFIX(msg)
-- end